#!/usr/bin/env python
# coding:utf-8
""" 
@author: nivic ybyang7
@license: Apache Licence 
@file: train.py
@time: 2023/09/06
@contact: ybyang7@iflytek.com
@site:  
@software: PyCharm 

# code is far away from bugs with the god animal protecting
    I love animals. They taste delicious.
              ┏┓      ┏┓
            ┏┛┻━━━┛┻┓
            ┃      ☃      ┃
            ┃  ┳┛  ┗┳  ┃
            ┃      ┻      ┃
            ┗━┓      ┏━┛
                ┃      ┗━━━┓
                ┃  神兽保佑    ┣┓
                ┃　永无BUG！   ┏┛
                ┗┓┓┏━┳┓┏┛
                  ┃┫┫  ┃┫┫
                  ┗┻┛  ┗┻┛ 
"""

#  Copyright (c) 2022. Lorem ipsum dolor sit amet, consectetur adipiscing elit.
#  Morbi non lorem porttitor neque feugiat blandit. Ut vitae ipsum eget quam lacinia accumsan.
#  Etiam sed turpis ac ipsum condimentum fringilla. Maecenas magna.
#  Proin dapibus sapien vel ante. Aliquam erat volutpat. Pellentesque sagittis ligula eget metus.
#  Vestibulum commodo. Ut rhoncus gravida arcu.
import subprocess
import os
from ailab.examples.model_imp import  train

trainfile = train.__file__
def start_train():
    model_name = os.environ.get("MODEL_NAME", "")
    dataset_path = os.environ.get("DATASET_PATH")
    pretrained_model_path = os.environ.get("PRETRAINED_MODEL_PATH", f"/home/.atp/models/chinese_llama_alpaca_2")
    tokenizer_path = os.environ.get("TOKENIZER_PATH", "/home/.atp/models/chinese_llama_alpaca_2")
    epochs = os.environ.get("EPOCHS", 4)
    lr = os.environ.get("LEARNING_RATE", 5e-5)
    opdir = os.environ.get("OUTPUT_DIR", f"/work/train_output/{model_name}")
    env = {
        "PRETRAINED_MODEL_NAME": "chinese_llama_alpaca_2",
        "MODEL_NAME": model_name,
        "DATASET_PATH": dataset_path,
        "PRETRAINED_MODEL_PATH": pretrained_model_path,
        "TOKENIZER_PATH": tokenizer_path,
        "OUTPUT_DIR": opdir,
        "EPOCHS": str(epochs),
        "LEARNING_RATE": str(lr),

    }
    os.environ.update(env)

    for k,v in env.items():
        print(f"export {k}={v}")

    cmd = f'python -m ailab.examples.model_imp.train'
    ret = subprocess.call(cmd, shell=True, env=dict(os.environ))
    return ret


if __name__ == '__main__':
    exit(start_train())



import os, argparse
import torch
from ailab.atp_dataset.dataset import AILabDataset
from ailab.atp_dataset.constant import Sources
from ailab.atp_finetuner.constant import Task, Framework
from ailab.atp_finetuner.finetuner import AILabFinetuner
from ailab.atp_finetuner.constant import Model


def train_progress(percent: float):
    pass


def model_test(args):
    # todo     # fixed pretrained in train_deprecatied.py
    pretrained_model_name = args.PRETRAINED_MODEL_NAME

    dataset_path = args.DATASET_PATH
    output_dir = args.OUTPUT_DIR
    pretrained_model_path = args.PRETRAINED_MODEL_PATH
    tokenizer_path = args.TOKENIZER_PATH
    finetune_type = args.FINETUNE_TYPE
    epoch = int(args.EPOCHS)
    learning_rate = float(args.LEARNING_RATE)
    max_source_length = int(args.MAX_SOURCE_LENGTH)
    end_to_zip = True if args.END_TO_ZIP.lower() == 'true' else False

    args = {
        "model_args": {
            "quantization_bit": None,  # LoRA
            "max_source_length": max_source_length,
        },
        "train_args": {
            "output_dir": output_dir,
            "evaluation_strategy": "epoch",
            "per_device_eval_batch_size": 4,
            "eval_steps": 250,
            "resume_from_checkpoint": False,
            "num_train_epochs": epoch,
            "per_device_train_batch_size": 4,
            "gradient_accumulation_steps": 4,
            "learning_rate": learning_rate,
            "logging_steps": 10,
            "save_steps": 500,
            "bf16": False,
            "save_strategy": "steps",
            "weight_decay": 0,
            "end_to_zip":end_to_zip
        },
    }

    # 根据不同的模型，个性化处理
    dataset = AILabDataset.load_dataset(dataset_path, src=Sources.huggingface)

    if pretrained_model_name in [
        Model.chinese_alpaca,
        Model.chinese_alpaca_2,
        Model.vicuna,
        Model.open_llama,
        Model.alpaca,
        Model.ziya_llama_13b,
        Model.bencao_llama,
    ]:
        dataset.train_test_split(test_size=0.2)

    if pretrained_model_name == Model.chinese_alpaca_2:
        finetune_type == "lora"

    if pretrained_model_name == Model.xverse_13b or pretrained_model_name == Model.code_geex_2:
        args["train_args"]["bf16"] = True

    # 针对 微调方式的处理
    if finetune_type == "qlora":
        args["model_args"]["quantization_bit"] = 4
        args["model_args"]["quantization_type"] = "nf4"
        args["model_args"]["double_quantization"] = True
        args["model_args"]["compute_dtype"] = torch.float16

    finetuner = AILabFinetuner(
        Task.question_answering,
        Framework.Pytorch,
        dataset,
        pretrained_model_name,
        train_progress,
        pretrained_model_path,
        tokenizer_path,
        **args
    )
    finetuner.finetuner()


if __name__ == "__main__":
    # 创建 ArgumentParser 对象
    parser = argparse.ArgumentParser(description="Example script with arguments")

    # 添加参数
    parser.add_argument("--PRETRAINED_MODEL_NAME", type=str, help="Pretrained model name")
    parser.add_argument("--DATASET_PATH", type=str, help="Dataset path")
    parser.add_argument("--PRETRAINED_MODEL_PATH", type=str, help="Pretrained model path")
    parser.add_argument("--TOKENIZER_PATH", type=str, help="Tokenizer path")
    parser.add_argument("--OUTPUT_DIR", type=str, help="Output directory")
    parser.add_argument("--FINETUNE_TYPE", type=str, help="Finetune type")
    parser.add_argument("--EPOCHS", type=str, help="epochs")
    parser.add_argument("--LEARNING_RATE", type=str, help="learning_rate")
    parser.add_argument("--MAX_SOURCE_LENGTH", type=str, help="max source token length")
    parser.add_argument("--END_TO_ZIP", type=str, help="after train to pack zip")

    # 解析命令行参数
    args = parser.parse_args()

    model_test(args)



import os
from ailab.atp_evaluation.constant import BenchMarkType, Model
from ailab.atp_evaluation.evaluator import AILabEvaluator

# 测试数据集本地路径
DATASET_DIR:str = "/home/sdk_dataset/ailabsdk_dataset/evaluation"
OUTDIR:str = "./result"

def is_chinese_task(tasktype:BenchMarkType) -> bool:
    if (tasktype == BenchMarkType.ceval
        or tasktype == BenchMarkType.ceval_validation
        or tasktype == BenchMarkType.cmmlu
        or tasktype == BenchMarkType.gaokao):
        return True
    return False

def llama_7b_test(tasktype:BenchMarkType, use_peft:bool = False, gpu_index:int = 0):
    assert use_peft==False, "only test for llama_7b base model"
    lora_weight=None
    base_model="/home/sdk_models/llama-7b-hf"
    tokenizer_dir='/home/sdk_token/llama-7b-hf_tokenizer'
    evaluator = AILabEvaluator(tasktype,
                            model_name=Model.llama_7b,
                            model_dir=base_model,
                            dataset_dir=DATASET_DIR,
                            tokenizer_dir=tokenizer_dir,
                            lora_weight_dir=lora_weight,
                            output_dir=OUTDIR,
                            cudaIdx=gpu_index)
    evaluator.evaluate()

def stanford_alpaca_test(tasktype:BenchMarkType, use_peft:bool = True, gpu_index:int = 0):
    assert use_peft, "to test llama-7b-hf base model, please use llama_7b_test api, and set use_peft to False"
    lora_weight="/home/finetuned_models/my_standford_alpaca_model" if use_peft else None
    base_model="/home/sdk_models/llama-7b-hf"
    tokenizer_dir='/home/sdk_token/llama-7b-hf_tokenizer'
    evaluator = AILabEvaluator(tasktype,
                            model_name=Model.alpaca,
                            model_dir=base_model,
                            dataset_dir=DATASET_DIR,
                            tokenizer_dir=tokenizer_dir,
                            lora_weight_dir=lora_weight,
                            output_dir=OUTDIR,
                            cudaIdx=gpu_index)
    evaluator.evaluate()

def vicuna_test(tasktype:BenchMarkType, use_peft:bool = True, gpu_index:int = 0):
    assert use_peft, "to test llama-7b-hf base model, please use llama_7b_test api, and set use_peft to False"
    lora_weight="/home/finetuned_models/my_chinese_llama_vicuna_model" if use_peft else None
    base_model="/home/sdk_models/llama-7b-hf"
    tokenizer_dir='/home/sdk_token/chinese_llama_vicuna_tokenizer'
    evaluator = AILabEvaluator(tasktype,
                            model_name=Model.vicuna,
                            model_dir=base_model,
                            dataset_dir=DATASET_DIR,
                            tokenizer_dir=tokenizer_dir,
                            lora_weight_dir=lora_weight,
                            output_dir=OUTDIR,
                            cudaIdx=gpu_index)
    evaluator.evaluate()

def chinese_alpaca_test(tasktype:BenchMarkType, use_peft:bool = True, gpu_index:int = 0):
    assert use_peft, "to test llama-7b-hf base model, please use llama_7b_test api, and set use_peft to False"
    lora_weight="/home/finetuned_models/my_chinese_llama_alpaca_model" if use_peft else None
    base_model="/home/sdk_models/llama-7b-hf"
    tokenizer_dir='/home/sdk_token/chinese_llama_alpaca_tokenizer'
    evaluator = AILabEvaluator(tasktype,
                            model_name=Model.chinese_alpaca,
                            model_dir=base_model,
                            dataset_dir=DATASET_DIR,
                            tokenizer_dir=tokenizer_dir,
                            lora_weight_dir=lora_weight,
                            output_dir=OUTDIR,
                            cudaIdx=gpu_index)
    evaluator.evaluate()

def chatglm_6b_test(tasktype:BenchMarkType, use_peft:bool = False, gpu_index:int = 0):
    lora_weight="/home/finetuned_models/my_chatglm_6b_model" if use_peft else None
    base_model="/home/sdk_models/chatglm-6b"
    tokenizer_dir=None
    evaluator = AILabEvaluator(tasktype,
                            model_name=Model.chatglm_6b,
                            model_dir=base_model,
                            dataset_dir=DATASET_DIR,
                            tokenizer_dir=tokenizer_dir,
                            lora_weight_dir=lora_weight,
                            output_dir=OUTDIR,
                            cudaIdx=gpu_index)
    evaluator.evaluate()

def chatglm2_6b_test(tasktype:BenchMarkType, use_peft:bool = False, gpu_index:int = 0):
    lora_weight="/home/finetuned_models/my_chatglm2_model" if use_peft else None
    base_model="/home/sdk_models/chatglm2_6b"
    tokenizer_dir=None
    evaluator = AILabEvaluator(tasktype,
                            model_name=Model.chatglm2_6b,
                            model_dir=base_model,
                            dataset_dir=DATASET_DIR,
                            tokenizer_dir=tokenizer_dir,
                            lora_weight_dir=lora_weight,
                            output_dir=OUTDIR,
                            cudaIdx=gpu_index)
    evaluator.evaluate()

def baichuan_7b_test(tasktype:BenchMarkType, use_peft:bool = False, gpu_index:int = 0):
    lora_weight="/home/finetuned_models/my_baichuan_model" if use_peft else None
    base_model="/home/sdk_models/baichuan_7b"
    tokenizer_dir=None
    evaluator = AILabEvaluator(tasktype,
                            model_name=Model.baichuan_7b,
                            model_dir=base_model,
                            dataset_dir=DATASET_DIR,
                            tokenizer_dir=tokenizer_dir,
                            lora_weight_dir=lora_weight,
                            output_dir=OUTDIR,
                            cudaIdx=gpu_index)
    evaluator.evaluate()

def open_llama_test(tasktype:BenchMarkType, use_peft:bool = False, gpu_index:int = 0):
    lora_weight="/home/finetuned_models/my_open_llama_model" if use_peft else None
    base_model="/home/sdk_models/open_llama_7b"
    tokenizer_dir=None
    evaluator = AILabEvaluator(tasktype,
                            model_name=Model.open_llama,
                            model_dir=base_model,
                            dataset_dir=DATASET_DIR,
                            tokenizer_dir=tokenizer_dir,
                            lora_weight_dir=lora_weight,
                            output_dir=OUTDIR,
                            cudaIdx=gpu_index)
    evaluator.evaluate()

def baichuan_13b_test(tasktype:BenchMarkType, use_peft:bool = False, gpu_index:int = 0):
    lora_weight="/home/finetuned_models/my_baichuan13b_model" if use_peft else None
    base_model="/home/sdk_models/baichuan_13b"
    tokenizer_dir=None
    evaluator = AILabEvaluator(tasktype,
                            model_name=Model.baichuan_13b,
                            model_dir=base_model,
                            dataset_dir=DATASET_DIR,
                            tokenizer_dir=tokenizer_dir,
                            lora_weight_dir=lora_weight,
                            output_dir=OUTDIR,
                            cudaIdx=gpu_index)
    evaluator.evaluate()

def bloomz_7b1_mt_test(tasktype:BenchMarkType, use_peft:bool = False, gpu_index:int = 0):
    lora_weight="/home/finetuned_models/my_bloomz_model" if use_peft else None
    base_model="/home/sdk_models/bloomz_7b"
    tokenizer_dir=None
    evaluator = AILabEvaluator(tasktype,
                            model_name=Model.bloomz_7b1_mt,
                            model_dir=base_model,
                            dataset_dir=DATASET_DIR,
                            tokenizer_dir=tokenizer_dir,
                            lora_weight_dir=lora_weight,
                            output_dir=OUTDIR,
                            cudaIdx=gpu_index)
    evaluator.evaluate()

def falcon_7b_test(tasktype:BenchMarkType, use_peft:bool = False, gpu_index:int = 0):
    lora_weight="/home/finetuned_models/my_falcon_model" if use_peft else None
    base_model="/home/sdk_models/falcon_7b"
    tokenizer_dir=None
    evaluator = AILabEvaluator(tasktype,
                            model_name=Model.falcon_7b,
                            model_dir=base_model,
                            dataset_dir=DATASET_DIR,
                            tokenizer_dir=tokenizer_dir,
                            lora_weight_dir=lora_weight,
                            output_dir=OUTDIR,
                            cudaIdx=gpu_index)
    evaluator.evaluate()

def moss_moon_003_base_test(tasktype:BenchMarkType, use_peft:bool = False, gpu_index:int = 0):
    lora_weight="/home/finetuned_models/my_moss_model" if use_peft else None
    base_model="/home/sdk_models/moss-moon-003-base"
    tokenizer_dir=None
    evaluator = AILabEvaluator(tasktype,
                            model_name=Model.moss_moon_003_base,
                            model_dir=base_model,
                            dataset_dir=DATASET_DIR,
                            tokenizer_dir=tokenizer_dir,
                            lora_weight_dir=lora_weight,
                            output_dir=OUTDIR,
                            cudaIdx=gpu_index)
    evaluator.evaluate()

def llama2_7b_test(tasktype:BenchMarkType, use_peft:bool = False, gpu_index:int = 0):
    lora_weight="/home/finetuned_models/my_llama2_model" if use_peft else None
    base_model="/home/sdk_models/llama2-7b-hf"
    tokenizer_dir=None
    evaluator = AILabEvaluator(tasktype,
                            model_name=Model.llama2_7b,
                            model_dir=base_model,
                            dataset_dir=DATASET_DIR,
                            tokenizer_dir=tokenizer_dir,
                            lora_weight_dir=lora_weight,
                            output_dir=OUTDIR,
                            cudaIdx=gpu_index)
    evaluator.evaluate()

# def llama2_chinese_alpaca_test(tasktype:BenchMarkType, use_peft:bool = False, gpu_index:int = 0):
#     lora_weight="/home/finetuned_models/my_chinese_alpaca_2_model" if use_peft else None
#     base_model="/home/sdk_models/chinese_llama_alpaca_2"
#     tokenizer_dir=None
#     evaluator = AILabEvaluator(tasktype,
#                             model_name=Model.,
#                             model_dir=base_model,
#                             dataset_dir=DATASET_DIR,
#                             tokenizer_dir=tokenizer_dir,
#                             lora_weight_dir=lora_weight,
#                             output_dir=OUTDIR,
#                             cudaIdx=gpu_index)
#     evaluator.evaluate()

# for test all models in A800
def task1(tasktype:BenchMarkType, idx:int = 0):
    vicuna_test(tasktype, use_peft = True, gpu_index = idx)
    baichuan_7b_test(tasktype, use_peft = True, gpu_index = idx)

def task2(tasktype:BenchMarkType, idx:int = 1):
    open_llama_test(tasktype, use_peft = False, gpu_index = idx)
    baichuan_7b_test(tasktype, use_peft = False, gpu_index = idx)

def task3(tasktype:BenchMarkType, idx:int = 2):
    chatglm_6b_test(tasktype, use_peft = True, gpu_index = idx)
    bloomz_7b1_mt_test(tasktype, use_peft = False, gpu_index = idx)
    bloomz_7b1_mt_test(tasktype, use_peft = True, gpu_index = idx)

def task4(tasktype:BenchMarkType, idx:int = 3):
    chatglm_6b_test(tasktype, use_peft = False, gpu_index = idx)
    chatglm2_6b_test(tasktype, use_peft = False, gpu_index = idx)
    chatglm2_6b_test(tasktype, use_peft = True, gpu_index = idx)

def task5(tasktype:BenchMarkType, idx:int = 4):
    falcon_7b_test(tasktype, use_peft = False, gpu_index = idx)
    falcon_7b_test(tasktype, use_peft = True, gpu_index = idx)
    open_llama_test(tasktype, use_peft = True, gpu_index = idx)

def task6(tasktype:BenchMarkType, idx:int = 5):
    llama_7b_test(tasktype, use_peft = False, gpu_index = idx)
    stanford_alpaca_test(tasktype, use_peft = True, gpu_index = idx)
    chinese_alpaca_test(tasktype, use_peft = True, gpu_index = idx)

def task7(tasktype:BenchMarkType, idx:int = 6):
    llama2_7b_test(tasktype, use_peft = False, gpu_index = idx)
    moss_moon_003_base_test(tasktype, use_peft = False, gpu_index = idx)
    moss_moon_003_base_test(tasktype, use_peft = True, gpu_index = idx)

def task8(tasktype:BenchMarkType, idx:int = 7):
    llama2_7b_test(tasktype, use_peft = True, gpu_index = idx)
    baichuan_13b_test(tasktype, use_peft = False, gpu_index = idx)
    baichuan_13b_test(tasktype, use_peft = True, gpu_index = idx)

if __name__ == '__main__':
    chatglm_6b_test(tasktype = BenchMarkType.truthfulqa_mc, use_peft = False, gpu_index = 3)


 




import os
import torch
from ailab.atp_dataset.dataset import AILabDataset
from ailab.atp_dataset.constant import Sources
from ailab.atp_finetuner.constant import Task, Framework
from ailab.atp_finetuner.finetuner import AILabFinetuner

def train_progress(percent: float):
    pass


def chinese_alpaca_test():
    # todo     # fixed pretrained in train_deprecatied.py
    pretrained_model_name = os.environ.get("PRETRAINED_MODEL_NAME","chinese_llama_alpaca_2")
    model_name = os.environ.get("MODEL_NAME","")
    dataset_path = os.environ.get("DATASET_PATH")
    output_dir = os.environ.get("OUTPUT_DIR", f"/work/train_output/{model_name}")
    pretrained_model_path = os.environ.get("PRETRAINED_MODEL_PATH", f"/home/.atp/models/chinese_llama_alpaca_2")
    tokenizer_path = os.environ.get("TOKENIZER_PATH", "/home/.atp/models/chinese_llama_alpaca_2")
    if not dataset_path or not tokenizer_path:
        raise TypeError(
            f'os.environ should have (DATASET_PATH,TOKENIZER_PATH)')

    dataset = AILabDataset.load_dataset(dataset_path, src=Sources.huggingface)
    dataset.train_test_split(test_size=0.2)
    args = {
        "model_args": {
            "quantization_bit": None, # LoRA
        },
        "train_args": {
            "output_dir": output_dir,
            "evaluation_strategy": "epoch",
            "save_strategy": "steps",
            "learning_rate": float(os.environ.get("LEARNING_RATE",  1e-4)),
            "per_device_train_batch_size": 4,
            "gradient_accumulation_steps": 4,
            "per_device_eval_batch_size": 4,
            "num_train_epochs": int(os.environ.get("NUM_TRAIN_EPOCHS",  3)),
            "weight_decay": 0.0,
            "logging_steps": 10,
            "fp16": True,
            "bf16": False,
            "eval_steps": 250,
            "save_steps": 500,
            "resume_from_checkpoint": False,
        },
    }
    finetuner = AILabFinetuner(Task.question_answering, Framework.Pytorch, dataset,
                               pretrained_model_name, train_progress,
                               pretrained_model_path,
                               tokenizer_path,
                               **args)
    finetuner.finetuner()


if __name__ == '__main__':
    chinese_alpaca_test()



# YOLOv5 🚀 by Ultralytics, GPL-3.0 license

# WARNING ⚠️ wandb is deprecated and will be removed in future release.
# See supported integrations at https://github.com/ultralytics/yolov5#integrations

import logging
import os
import sys
from contextlib import contextmanager
from pathlib import Path

from utils.general import LOGGER, colorstr

FILE = Path(__file__).resolve()
ROOT = FILE.parents[3]  # YOLOv5 root directory
if str(ROOT) not in sys.path:
    sys.path.append(str(ROOT))  # add ROOT to PATH
RANK = int(os.getenv('RANK', -1))
DEPRECATION_WARNING = f"{colorstr('wandb')}: WARNING ⚠️ wandb is deprecated and will be removed in a future release. " \
                      f'See supported integrations at https://github.com/ultralytics/yolov5#integrations.'

try:
    import wandb

    assert hasattr(wandb, '__version__')  # verify package import not local dir
    LOGGER.warning(DEPRECATION_WARNING)
except (ImportError, AssertionError):
    wandb = None


class WandbLogger():
    """Log training runs, datasets, models, and predictions to Weights & Biases.

    This logger sends information to W&B at wandb.ai. By default, this information
    includes hyperparameters, system configuration and metrics, model metrics,
    and basic data metrics and analyses.

    By providing additional command line arguments to train_deprecatied.py, datasets,
    models and predictions can also be logged.

    For more on how this logger is used, see the Weights & Biases documentation:
    https://docs.wandb.com/guides/integrations/yolov5
    """

    def __init__(self, opt, run_id=None, job_type='Training'):
        """
        - Initialize WandbLogger instance
        - Upload dataset if opt.upload_dataset is True
        - Setup training processes if job_type is 'Training'

        arguments:
        opt (namespace) -- Commandline arguments for this run
        run_id (str) -- Run ID of W&B run to be resumed
        job_type (str) -- To set the job_type for this run

       """
        # Pre-training routine --
        self.job_type = job_type
        self.wandb, self.wandb_run = wandb, wandb.run if wandb else None
        self.val_artifact, self.train_artifact = None, None
        self.train_artifact_path, self.val_artifact_path = None, None
        self.result_artifact = None
        self.val_table, self.result_table = None, None
        self.max_imgs_to_log = 16
        self.data_dict = None
        if self.wandb:
            self.wandb_run = wandb.init(config=opt,
                                        resume='allow',
                                        project='YOLOv5' if opt.project == 'runs/train' else Path(opt.project).stem,
                                        entity=opt.entity,
                                        name=opt.name if opt.name != 'exp' else None,
                                        job_type=job_type,
                                        id=run_id,
                                        allow_val_change=True) if not wandb.run else wandb.run

        if self.wandb_run:
            if self.job_type == 'Training':
                if isinstance(opt.data, dict):
                    # This means another dataset manager has already processed the dataset info (e.g. ClearML)
                    # and they will have stored the already processed dict in opt.data
                    self.data_dict = opt.data
                self.setup_training(opt)

    def setup_training(self, opt):
        """
        Setup the necessary processes for training YOLO models:
          - Attempt to download model checkpoint and dataset artifacts if opt.resume stats with WANDB_ARTIFACT_PREFIX
          - Update data_dict, to contain info of previous run if resumed and the paths of dataset artifact if downloaded
          - Setup log_dict, initialize bbox_interval

        arguments:
        opt (namespace) -- commandline arguments for this run

        """
        self.log_dict, self.current_epoch = {}, 0
        self.bbox_interval = opt.bbox_interval
        if isinstance(opt.resume, str):
            model_dir, _ = self.download_model_artifact(opt)
            if model_dir:
                self.weights = Path(model_dir) / 'last.pt'
                config = self.wandb_run.config
                opt.weights, opt.save_period, opt.batch_size, opt.bbox_interval, opt.epochs, opt.hyp, opt.imgsz = str(
                    self.weights), config.save_period, config.batch_size, config.bbox_interval, config.epochs, \
                    config.hyp, config.imgsz

        if opt.bbox_interval == -1:
            self.bbox_interval = opt.bbox_interval = (opt.epochs // 10) if opt.epochs > 10 else 1
            if opt.evolve or opt.noplots:
                self.bbox_interval = opt.bbox_interval = opt.epochs + 1  # disable bbox_interval

    def log_model(self, path, opt, epoch, fitness_score, best_model=False):
        """
        Log the model checkpoint as W&B artifact

        arguments:
        path (Path)   -- Path of directory containing the checkpoints
        opt (namespace) -- Command line arguments for this run
        epoch (int)  -- Current epoch number
        fitness_score (float) -- fitness score for current epoch
        best_model (boolean) -- Boolean representing if the current checkpoint is the best yet.
        """
        model_artifact = wandb.Artifact('run_' + wandb.run.id + '_model',
                                        type='model',
                                        metadata={
                                            'original_url': str(path),
                                            'epochs_trained': epoch + 1,
                                            'save period': opt.save_period,
                                            'project': opt.project,
                                            'total_epochs': opt.epochs,
                                            'fitness_score': fitness_score})
        model_artifact.add_file(str(path / 'last.pt'), name='last.pt')
        wandb.log_artifact(model_artifact,
                           aliases=['latest', 'last', 'epoch ' + str(self.current_epoch), 'best' if best_model else ''])
        LOGGER.info(f'Saving model artifact on epoch {epoch + 1}')

    def val_one_image(self, pred, predn, path, names, im):
        pass

    def log(self, log_dict):
        """
        save the metrics to the logging dictionary

        arguments:
        log_dict (Dict) -- metrics/media to be logged in current step
        """
        if self.wandb_run:
            for key, value in log_dict.items():
                self.log_dict[key] = value

    def end_epoch(self):
        """
        commit the log_dict, model artifacts and Tables to W&B and flush the log_dict.

        arguments:
        best_result (boolean): Boolean representing if the result of this evaluation is best or not
        """
        if self.wandb_run:
            with all_logging_disabled():
                try:
                    wandb.log(self.log_dict)
                except BaseException as e:
                    LOGGER.info(
                        f'An error occurred in wandb logger. The training will proceed without interruption. More info\n{e}'
                    )
                    self.wandb_run.finish()
                    self.wandb_run = None
                self.log_dict = {}

    def finish_run(self):
        """
        Log metrics if any and finish the current W&B run
        """
        if self.wandb_run:
            if self.log_dict:
                with all_logging_disabled():
                    wandb.log(self.log_dict)
            wandb.run.finish()
            LOGGER.warning(DEPRECATION_WARNING)


@contextmanager
def all_logging_disabled(highest_level=logging.CRITICAL):
    """ source - https://gist.github.com/simon-weber/7853144
    A context manager that will prevent any logging messages triggered during the body from being processed.
    :param highest_level: the maximum logging level in use.
      This would only need to be changed if a custom level greater than CRITICAL is defined.
    """
    previous_level = logging.root.manager.disable
    logging.disable(highest_level)
    try:
        yield
    finally:
        logging.disable(previous_level)



import os
import subprocess
import sys
from ailab.log import logger


def install_requiremet(basedir: str):
    basereqs = os.path.join(basedir, "requirements.txt")
    if not os.path.exists(basereqs):
        return
    cmd = "pip install -r  " + basereqs
    print(f"executing {cmd}")

    subprocess.call(cmd, shell=True)


from typing import Dict, List, Optional
import os, json, torch
from transformers.trainer import TRAINER_STATE_NAME

VALUE_HEAD_FILE_NAME = "value_head.bin"


def get_state_dict(model: torch.nn.Module) -> Dict[str, torch.Tensor]:  # get state dict containing trainable parameters
    state_dict = model.state_dict()
    filtered_state_dict = {}

    for k, v in model.named_parameters():
        if v.requires_grad:
            filtered_state_dict[k] = state_dict[k].cpu().clone().detach()
    return filtered_state_dict


def load_trainable_params(model: torch.nn.Module, checkpoint_dir: os.PathLike) -> bool:
    from peft.utils.other import WEIGHTS_NAME
    weights_file = os.path.join(checkpoint_dir, WEIGHTS_NAME)
    if not os.path.exists(weights_file):
        logger.info("Provided path ({}) does not contain pre-trained weights.".format(checkpoint_dir))
        return False
    model_state_dict = torch.load(weights_file, map_location="cpu")
    model.load_state_dict(model_state_dict, strict=False)  # skip missing keys
    return True


def load_valuehead_params(model: torch.nn.Module, checkpoint_dir: os.PathLike) -> bool:
    valuehead_file = os.path.join(checkpoint_dir, VALUE_HEAD_FILE_NAME)
    if not os.path.exists(valuehead_file):
        logger.info("Provided path ({}) does not contain valuehead weights.".format(checkpoint_dir))
        return False
    valuehead_state_dict = torch.load(valuehead_file, map_location="cpu")
    model.register_buffer("reward_head_weight", valuehead_state_dict["summary.weight"])
    model.register_buffer("reward_head_bias", valuehead_state_dict["summary.bias"])
    model.register_buffer("default_head_weight", torch.zeros_like(valuehead_state_dict["summary.weight"]))
    model.register_buffer("default_head_bias", torch.zeros_like(valuehead_state_dict["summary.bias"]))
    return True


def smooth(scalars: List[float], weight: Optional[float] = 0.9) -> List[float]:
    r"""
    EMA implementation according to TensorBoard.
    """
    last = scalars[0]
    smoothed = list()
    for next_val in scalars:
        smoothed_val = last * weight + (1 - weight) * next_val
        smoothed.append(smoothed_val)
        last = smoothed_val
    return smoothed


def plot_loss(save_dictionary: os.PathLike, keys: Optional[List[str]] = ["loss"]) -> None:
    import matplotlib.pyplot as plt
    with open(os.path.join(save_dictionary, TRAINER_STATE_NAME), "r", encoding="utf-8") as f:
        data = json.load(f)

    for key in keys:
        steps, metrics = [], []
        for i in range(len(data["log_history"])):
            if key in data["log_history"][i]:
                steps.append(data["log_history"][i]["step"])
                metrics.append(data["log_history"][i][key])

        if len(metrics) == 0:
            logger.info(f"No metric {key} to plot.")
            continue

        plt.figure()
        plt.plot(steps, metrics, alpha=0.4, label="original")
        plt.plot(steps, smooth(metrics), label="smoothed")
        plt.title("training {} of {}".format(key, save_dictionary))
        plt.xlabel("step")
        plt.ylabel(key)
        plt.legend()
        plt.savefig(os.path.join(save_dictionary, "training_{}.png".format(key)), format="png", dpi=100)
        logger.info("Figure saved:", os.path.join(save_dictionary, "training_{}.png".format(key)))


import os
import zipfile
import shutil


def create_zip_and_delete_folder(folder_path, zip_path):
    try:
        # 删除 checkpoint-* 文件
        for root, dirs, _ in os.walk(folder_path):
            for dir_name in dirs[:]:
                if dir_name.startswith("checkpoint-"):
                    dir_to_remove = os.path.join(root, dir_name)
                    shutil.rmtree(dir_to_remove)
                    print(f"已删除文件夹: {dir_to_remove}")

        # 删除 runs 文件夹
        runs_folder = os.path.join(folder_path, "runs")
        if os.path.exists(runs_folder):
            shutil.rmtree(runs_folder)

        # 创建 Zip 文件
        with zipfile.ZipFile(zip_path, 'w', zipfile.ZIP_DEFLATED) as zipf:
            for root, dirs, files in os.walk(folder_path):
                for file in files:
                    file_path = os.path.join(root, file)
                    if file_path.endswith(".zip"):
                        continue
                    rel_path = os.path.relpath(file_path, folder_path)
                    zipf.write(file_path, rel_path)

        # 删除原始文件夹
        shutil.rmtree(folder_path)
        print(f"文件夹 '{folder_path}' 已打包为 '{zip_path}' 并删除原始文件夹")
    except Exception as e:
        print(f"发生错误：{e}")
